@@ -876,8 +876,11 @@ bool samplv1_jack_application::parse_args (void)
 		}
 		else
 		if (sArg == "-v" || sArg == "-V" || sArg == "--version") {
-			out << QString("Qt: %1\n")
-				.arg(qVersion());
+			out << QString("Qt: %1").arg(qVersion());
+		#if defined(QT_STATIC)
+			out << "-static";
+		#endif
+			out << '\n';
 			out << QString("%1: %2\n")
 				.arg(SAMPLV1_TITLE)
 				.arg(CONFIG_BUILD_VERSION);
