@@ -274,17 +274,36 @@ uint32_t samplv1widget_sample::loopEnd (void) const
 }
 
 
-// Sanitizer helper;
+// Sanitizer helper.
 int samplv1widget_sample::safeX ( int x ) const
 {
-	if (x < 0)
+	const int w = QFrame::width();
+	return (x < w ? x : w);
+}
+
+
+// Sanitized converters.
+int samplv1widget_sample::pixelFromFrames ( uint32_t n ) const
+{
+	const uint32_t nframes = m_pSample->length();
+	if (nframes == 0)
 		return 0;
 
 	const int w = QFrame::width();
-	if (x > w)
-		return w;
-	else
-		return x;
+	const int x = (uint64_t(w) * uint64_t(n)) / uint64_t(nframes);
+	return (x < w ? x : w);
+}
+
+
+uint32_t samplv1widget_sample::framesFromPixel ( int x ) const
+{
+	const int w = QFrame::width();
+	if (w == 0)
+		return 0;
+
+	const uint32_t nframes = m_pSample->length();
+	const uint32_t n = (uint64_t(x) * uint64_t(nframes)) / uint64_t(w);
+	return (n < nframes ? n : nframes);
 }
 
 
@@ -303,17 +322,14 @@ void samplv1widget_sample::mousePressEvent ( QMouseEvent *pMouseEvent )
 			m_dragState = DragStart;
 			m_posDrag = pMouseEvent->pos();
 		} else {
-			const uint32_t nframes = m_pSample->length();
-			if (nframes > 0 && (m_bOffset || m_bLoop)) {
-				const int w = QFrame::width();
-				if (m_bOffset) {
-					m_iDragOffsetStartX = safeX((m_iOffsetStart * w) / nframes);
-					m_iDragOffsetEndX   = safeX((m_iOffsetEnd   * w) / nframes);
-				}
-				if (m_bLoop) {
-					m_iDragLoopStartX = safeX((m_iLoopStart * w) / nframes);
-					m_iDragLoopEndX   = safeX((m_iLoopEnd   * w) / nframes);
-				}
+			if (m_bOffset) {
+				m_iDragOffsetStartX = pixelFromFrames(m_iOffsetStart);
+				m_iDragOffsetEndX   = pixelFromFrames(m_iOffsetEnd);
+				m_dragState = m_dragCursor;
+			}
+			if (m_bLoop) {
+				m_iDragLoopStartX = pixelFromFrames(m_iLoopStart);
+				m_iDragLoopEndX   = pixelFromFrames(m_iLoopEnd);
 				m_dragState = m_dragCursor;
 			}
 		}
@@ -330,54 +346,54 @@ void samplv1widget_sample::mouseMoveEvent ( QMouseEvent *pMouseEvent )
 	switch (m_dragState) {
 	case DragNone:
 		if (m_pSample) {
-			const uint32_t nframes = m_pSample->length();
-			if (nframes > 0) {
-				const int w  = QFrame::width();
-				const int dx = QApplication::startDragDistance();
-				const int x1 = (m_iOffsetStart * w) / nframes;
-				const int x2 = (m_iOffsetEnd * w) / nframes;
-				const int x3 = (m_iLoopStart * w) / nframes;
-				const int x4 = (m_iLoopEnd * w) / nframes;
-				if (abs(x4 - x) < dx && m_bLoop) {
+			const int dx = QApplication::startDragDistance();
+			if (m_bLoop) {
+				int x0 = pixelFromFrames(m_iLoopEnd);
+				if (qAbs(x0 - x) < dx) {
 					m_dragCursor = DragLoopEnd;
 					QFrame::setCursor(QCursor(Qt::SizeHorCursor));
 					QToolTip::showText(
 						QCursor::pos(),
 						tr("Loop end: %1")
 							.arg(textFromValue(m_iLoopEnd)), this);
+					break;
 				}
-				else
-				if (abs(x3 - x) < dx && m_bLoop) {
+				x0 = pixelFromFrames(m_iLoopStart);
+				if (qAbs(x0 - x) < dx) {
 					m_dragCursor = DragLoopStart;
 					QFrame::setCursor(QCursor(Qt::SizeHorCursor));
 					QToolTip::showText(
 						QCursor::pos(),
 						tr("Loop start: %1")
 							.arg(textFromValue(m_iLoopStart)), this);
+					break;
 				}
-				else
-				if (abs(x2 - x) < dx && m_bOffset) {
+			}
+			if (m_bOffset) {
+				int x0 = pixelFromFrames(m_iOffsetEnd);
+				if (qAbs(x0 - x) < dx) {
 					m_dragCursor = DragOffsetEnd;
 					QFrame::setCursor(QCursor(Qt::SizeHorCursor));
 					QToolTip::showText(
 						QCursor::pos(),
 						tr("Offset end: %1")
 							.arg(textFromValue(m_iOffsetEnd)), this);
+					break;
 				}
-				else
-				if (abs(x1 - x) < dx && m_bOffset) {
+				x0 = pixelFromFrames(m_iOffsetStart);
+				if (qAbs(x0 - x) < dx) {
 					m_dragCursor = DragOffsetStart;
 					QFrame::setCursor(QCursor(Qt::SizeHorCursor));
 					QToolTip::showText(
 						QCursor::pos(),
 						tr("Offset start: %1")
 							.arg(textFromValue(m_iOffsetStart)), this);
+					break;
 				}
-				else
-				if (m_dragCursor != DragNone) {
-					m_dragCursor  = DragNone;
-					QFrame::unsetCursor();
-				}
+			}
+			if (m_dragCursor != DragNone) {
+				m_dragCursor  = DragNone;
+				QFrame::unsetCursor();
 			}
 		}
 		break;
@@ -389,16 +405,12 @@ void samplv1widget_sample::mouseMoveEvent ( QMouseEvent *pMouseEvent )
 			if (m_bLoop && m_iDragOffsetStartX > m_iDragLoopStartX)
 				m_iDragOffsetStartX = m_iDragLoopStartX;
 			update();
-			const int w = QFrame::width();
-			if (w > 0) {
-				const uint32_t nframes = m_pSample->length();
-				const uint32_t iOffsetStart
-					= (m_iDragOffsetStartX * nframes) / w;
-				QToolTip::showText(
-					QCursor::pos(),
-					tr("Offset start: %1")
-						.arg(textFromValue(iOffsetStart)), this);
-			}
+			const uint32_t iOffsetStart
+				= framesFromPixel(m_iDragOffsetStartX);
+			QToolTip::showText(
+				QCursor::pos(),
+				tr("Offset start: %1")
+					.arg(textFromValue(iOffsetStart)), this);
 		}
 		break;
 	case DragOffsetEnd:
@@ -409,16 +421,12 @@ void samplv1widget_sample::mouseMoveEvent ( QMouseEvent *pMouseEvent )
 			if (m_bLoop && m_iDragOffsetEndX < m_iDragLoopEndX)
 				m_iDragOffsetEndX = m_iDragLoopEndX;
 			update();
-			const int w = QFrame::width();
-			if (w > 0) {
-				const uint32_t nframes = m_pSample->length();
-				const uint32_t iOffsetEnd
-					= (m_iDragOffsetEndX * nframes) / w;
-				QToolTip::showText(
-					QCursor::pos(),
-					tr("Offset end: %1")
-						.arg(textFromValue(iOffsetEnd)), this);
-			}
+			const uint32_t iOffsetEnd
+				= framesFromPixel(m_iDragOffsetEndX);
+			QToolTip::showText(
+				QCursor::pos(),
+				tr("Offset end: %1")
+					.arg(textFromValue(iOffsetEnd)), this);
 		}
 		break;
 	case DragOffsetRange:
@@ -427,18 +435,22 @@ void samplv1widget_sample::mouseMoveEvent ( QMouseEvent *pMouseEvent )
 			const QRect& rect = QRect(m_posDrag, pMouseEvent->pos()).normalized();
 			m_iDragOffsetStartX = safeX(rect.left());
 			m_iDragOffsetEndX   = safeX(rect.right());
-			update();
-			const int w = QFrame::width();
-			if (w > 0) {
-				const uint32_t nframes      = m_pSample->length();
-				const uint32_t iOffsetStart = (m_iDragOffsetStartX * nframes) / w;
-				const uint32_t iOffsetEnd   = (m_iDragOffsetEndX   * nframes) / w;
-				QToolTip::showText(
-					QCursor::pos(),
-					tr("Offset start: %1, end: %2")
-						.arg(textFromValue(iOffsetStart))
-						.arg(textFromValue(iOffsetEnd)), this);
+			if (m_bLoop) {
+				if (m_iDragOffsetStartX > m_iDragLoopStartX)
+					m_iDragOffsetStartX = m_iDragLoopStartX;
+				if (m_iDragOffsetEndX < m_iDragLoopEndX)
+					m_iDragOffsetEndX = m_iDragLoopEndX;
 			}
+			update();
+			const uint32_t iOffsetStart
+				= framesFromPixel(m_iDragOffsetStartX);
+			const uint32_t iOffsetEnd
+				= framesFromPixel(m_iDragOffsetEndX);
+			QToolTip::showText(
+				QCursor::pos(),
+				tr("Offset start: %1, end: %2")
+					.arg(textFromValue(iOffsetStart))
+					.arg(textFromValue(iOffsetEnd)), this);
 		}
 		break;
 	case DragLoopStart:
@@ -449,16 +461,12 @@ void samplv1widget_sample::mouseMoveEvent ( QMouseEvent *pMouseEvent )
 			if (m_iDragLoopStartX > m_iDragLoopEndX)
 				m_iDragLoopStartX = m_iDragLoopEndX;
 			update();
-			const int w = QFrame::width();
-			if (w > 0) {
-				const uint32_t nframes = m_pSample->length();
-				const uint32_t iLoopStart
-					= (m_iDragLoopStartX * nframes) / w;
-				QToolTip::showText(
-					QCursor::pos(),
-					tr("Loop start: %1")
-						.arg(textFromValue(iLoopStart)), this);
-			}
+			const uint32_t iLoopStart
+				= framesFromPixel(m_iDragLoopStartX);
+			QToolTip::showText(
+				QCursor::pos(),
+				tr("Loop start: %1")
+					.arg(textFromValue(iLoopStart)), this);
 		}
 		break;
 	case DragLoopEnd:
@@ -469,16 +477,12 @@ void samplv1widget_sample::mouseMoveEvent ( QMouseEvent *pMouseEvent )
 			if (m_iDragLoopEndX < m_iDragLoopStartX)
 				m_iDragLoopEndX = m_iDragLoopStartX;
 			update();
-			const int w = QFrame::width();
-			if (w > 0) {
-				const uint32_t nframes = m_pSample->length();
-				const uint32_t iLoopEnd
-					= (m_iDragLoopEndX * nframes) / w;
-				QToolTip::showText(
-					QCursor::pos(),
-					tr("Loop end: %1")
-						.arg(textFromValue(iLoopEnd)), this);
-			}
+			const uint32_t iLoopEnd
+				= framesFromPixel(m_iDragLoopEndX);
+			QToolTip::showText(
+				QCursor::pos(),
+				tr("Loop end: %1")
+					.arg(textFromValue(iLoopEnd)), this);
 		}
 		break;
 	case DragLoopRange:
@@ -487,18 +491,24 @@ void samplv1widget_sample::mouseMoveEvent ( QMouseEvent *pMouseEvent )
 			const QRect& rect = QRect(m_posDrag, pMouseEvent->pos()).normalized();
 			m_iDragLoopStartX = safeX(rect.left());
 			m_iDragLoopEndX   = safeX(rect.right());
-			update();
-			const int w = QFrame::width();
-			if (w > 0) {
-				const uint32_t nframes    = m_pSample->length();
-				const uint32_t iLoopStart = (m_iDragLoopStartX * nframes) / w;
-				const uint32_t iLoopEnd   = (m_iDragLoopEndX   * nframes) / w;
-				QToolTip::showText(
-					QCursor::pos(),
-					tr("Loop start: %1, end: %2")
-						.arg(textFromValue(iLoopStart))
-						.arg(textFromValue(iLoopEnd)), this);
+			if (m_bOffset) {
+				if (m_iDragLoopStartX < m_iDragOffsetStartX)
+					m_iDragLoopStartX = m_iDragOffsetStartX;
+				if (m_iDragLoopEndX > m_iDragOffsetEndX)
+					m_iDragLoopEndX = m_iDragOffsetEndX;
+				if (m_iDragLoopStartX > m_iDragLoopEndX)
+					m_iDragLoopStartX = m_iDragLoopEndX;
 			}
+			update();
+			const uint32_t iLoopStart
+				= framesFromPixel(m_iDragLoopStartX);
+			const uint32_t iLoopEnd
+				= framesFromPixel(m_iDragLoopEndX);
+			QToolTip::showText(
+				QCursor::pos(),
+				tr("Loop start: %1, end: %2")
+					.arg(textFromValue(iLoopStart))
+					.arg(textFromValue(iLoopEnd)), this);
 		}
 		break;
 	case DragStart:
@@ -547,74 +557,44 @@ void samplv1widget_sample::mouseReleaseEvent ( QMouseEvent *pMouseEvent )
 	QFrame::mouseReleaseEvent(pMouseEvent);
 
 	switch (m_dragState) {
-	case DragOffsetStart: {
-		const int w = QFrame::width();
-		if (m_pSample && w > 0) {
-			const uint32_t nframes = m_pSample->length();
-			m_iOffsetStart = (m_iDragOffsetStartX * nframes) / w;
+	case DragOffsetStart:
+		if (m_pSample && m_iDragOffsetStartX < m_iDragOffsetEndX) {
+			m_iOffsetStart = framesFromPixel(m_iDragOffsetStartX);
 			emit offsetRangeChanged();
-			updateToolTip();
-			update();
 		}
 		break;
-	}
-	case DragOffsetEnd: {
-		const int w = QFrame::width();
-		if (m_pSample && w > 0) {
-			const uint32_t nframes = m_pSample->length();
-			m_iOffsetEnd = (m_iDragOffsetEndX * nframes) / w;
+	case DragOffsetEnd:
+		if (m_pSample && m_iDragOffsetStartX < m_iDragOffsetEndX) {
+			m_iOffsetEnd = framesFromPixel(m_iDragOffsetEndX);
 			emit offsetRangeChanged();
-			updateToolTip();
-			update();
 		}
 		break;
-	}
-	case DragOffsetRange: {
-		const int w = QFrame::width();
-		if (m_pSample && w > 0) {
-			const uint32_t nframes = m_pSample->length();
-			m_iOffsetStart = (m_iDragOffsetStartX * nframes) / w;
-			m_iOffsetEnd   = (m_iDragOffsetEndX   * nframes) / w;
+	case DragOffsetRange:
+		if (m_pSample && m_iDragOffsetStartX < m_iDragOffsetEndX) {
+			m_iOffsetStart = framesFromPixel(m_iDragOffsetStartX);
+			m_iOffsetEnd   = framesFromPixel(m_iDragOffsetEndX);
 			emit offsetRangeChanged();
-			updateToolTip();
-			update();
 		}
 		break;
-	}
-	case DragLoopStart: {
-		const int w = QFrame::width();
-		if (m_pSample && w > 0) {
-			const uint32_t nframes = m_pSample->length();
-			m_iLoopStart = (m_iDragLoopStartX * nframes) / w;
+	case DragLoopStart:
+		if (m_pSample && m_iDragLoopStartX < m_iDragLoopEndX) {
+			m_iLoopStart = framesFromPixel(m_iDragLoopStartX);
 			emit loopRangeChanged();
-			updateToolTip();
-			update();
 		}
 		break;
-	}
-	case DragLoopEnd: {
-		const int w = QFrame::width();
-		if (m_pSample && w > 0) {
-			const uint32_t nframes = m_pSample->length();
-			m_iLoopEnd = (m_iDragLoopEndX * nframes) / w;
+	case DragLoopEnd:
+		if (m_pSample && m_iDragLoopStartX < m_iDragLoopEndX) {
+			m_iLoopEnd = framesFromPixel(m_iDragLoopEndX);
 			emit loopRangeChanged();
-			updateToolTip();
-			update();
 		}
 		break;
-	}
-	case DragLoopRange: {
-		const int w = QFrame::width();
-		if (m_pSample && w > 0) {
-			const uint32_t nframes = m_pSample->length();
-			m_iLoopStart = (m_iDragLoopStartX * nframes) / w;
-			m_iLoopEnd   = (m_iDragLoopEndX   * nframes) / w;
+	case DragLoopRange:
+		if (m_pSample && m_iDragLoopStartX < m_iDragLoopEndX) {
+			m_iLoopStart = framesFromPixel(m_iDragLoopStartX);
+			m_iLoopEnd   = framesFromPixel(m_iDragLoopEndX);
 			emit loopRangeChanged();
-			updateToolTip();
-			update();
 		}
-		break;
-	}
+		// Fall thru...
 	default:
 		break;
 	}
@@ -623,6 +603,9 @@ void samplv1widget_sample::mouseReleaseEvent ( QMouseEvent *pMouseEvent )
 
 	m_pDragSample = nullptr;
 	resetDragState();
+
+	updateToolTip();
+	update();
 }
 
 
@@ -707,7 +690,6 @@ void samplv1widget_sample::paintEvent ( QPaintEvent *pPaintEvent )
 
 	if (m_pSample && m_ppPolyg) {
 		const bool bEnabled = isEnabled();
-		const uint32_t nframes = m_pSample->length();
 		const int w2 = (w << 1);
 		painter.setRenderHint(QPainter::Antialiasing, true);
 		// Loop point lines...
@@ -718,11 +700,9 @@ void samplv1widget_sample::paintEvent ( QPaintEvent *pPaintEvent )
 				m_dragState == DragLoopRange) {
 				x1 = m_iDragLoopStartX;
 				x2 = m_iDragLoopEndX;
-			}
-			else
-			if (nframes > 0) {
-				x1 = (m_iLoopStart * w) / nframes;
-				x2 = (m_iLoopEnd   * w) / nframes;
+			} else {
+				x1 = pixelFromFrames(m_iLoopStart);
+				x2 = pixelFromFrames(m_iLoopEnd);
 			}
 			QLinearGradient grad1(0, 0, w2, h);
 			painter.setPen(rgbLite);
@@ -758,11 +738,9 @@ void samplv1widget_sample::paintEvent ( QPaintEvent *pPaintEvent )
 				m_dragState == DragOffsetRange) {
 				x1 = m_iDragOffsetStartX;
 				x2 = m_iDragOffsetEndX;
-			}
-			else
-			if (nframes > 0) {
-				x1 = (m_iOffsetStart * w) / nframes;
-				x2 = (m_iOffsetEnd   * w) / nframes;
+			} else {
+				x1 = pixelFromFrames(m_iOffsetStart);
+				x2 = pixelFromFrames(m_iOffsetEnd);
 			}
 			QColor rgbOver = rgbDark.darker();
 			rgbOver.setAlpha(120);
